// Copyright (C) 2019 Checkmk GmbH - License: Check_MK Enterprise License
// This file is part of Checkmk (https://checkmk.com). It is subject to the
// terms and conditions defined in the file COPYING, which is part of this
// source code package.

#ifndef AlertHelper_h
#define AlertHelper_h

#include <cstdint>
#include <functional>
#include <string>
#include <string_view>

#include "cmc/ProcessHelper.h"
#include "cmc/Producer.h"

class AlertHandlerConfig;
enum class AlertHandlerEventType;
class EventSpooler;
class GeneralConfig;
class GlobalState;
class Logger;
class MonitoringLog;
class NotifyHelper;
class Object;
class Poller;

class AlertHelper : public ProcessHelper {
public:
    enum class Events { none, state_change, check_result, all };

    // Full featured CTOR
    AlertHelper(
        std::string name, bool enabled, bool enabled_global,
        Events events_to_process, NotifyHelper *notify,
        const std::function<Object *(const std::string &, const std::string &)>
            &get_response_object,
        MonitoringLog &monitoring_log, Logger *logger, EventSpooler &spooler);

    // Classic CTOR, used now by Core
    AlertHelper(
        std::string name, NotifyHelper *notify,
        const std::function<Object *(const std::string &, const std::string &)>
            &get_response_object,
        MonitoringLog &monitoring_log, Logger *logger, EventSpooler &spooler);

    ~AlertHelper() override;

    bool updateConfig(const GeneralConfig &general,
                      const AlertHandlerConfig &alert_handler,
                      const GlobalState &state);
    void tryToSendEvent();
    void handle(const Object &object, AlertHandlerEventType event_type);

private:
    bool _enabled;
    bool _enabled_global;
    uint32_t _event_types = 0;     // GlobalConfig::AlertHandlerEventType
    NotifyHelper *_notify_helper;  // output channel for alert helper. Ref.(?)
    // object owner. To parent class(?)
    std::function<Object *(const std::string &, const std::string &)>
        _get_response_object;
    MonitoringLog &monitoring_log_;
    ChunkProducer _stdout_consumer;
    bool _helper_ready;
    EventSpooler &spooler_;

    // AbstractHelper/ProcessHelper API
    void start_() override;
    void stop_() override;
    void setUpPoller_(Poller &poller) const override;
    void communicate_(const Poller &poller) override;

    void do_stop();

    void handleStdOut();
    void processResponse(std::string_view response);
    void processResponseStar();
    void processResponseSTARTED(std::string_view response);
    void processResponseSTOPPED(std::string_view response);

    static uint32_t decodeEvents(Events events_to_process);
};

#endif  // AlertHelper_h
